import os
import sys
import requests
import json
import webbrowser
from ricecooker import config
from ricecooker.classes import nodes, questions
from requests.exceptions import HTTPError
from ricecooker.managers import ChannelManager, RestoreManager, Status

def uploadchannel(path, debug, verbose=False, update=False, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, **kwargs):

    """ uploadchannel: Upload channel to Kolibri Studio server
        Args:
            path (str): path to file containing channel data
            debug (bool): determine which domain to upload to
            verbose (bool): indicates whether to print process (optional)
        Returns: (str) link to access newly created channel
    """
    # Get domain to upload to
    domain = config.PRODUCTION_DOMAIN
    if debug:
      domain = config.DEBUG_DOMAIN
    config.init_file_mapping_store(debug)

    config.init_file_mapping_store(debug)

    # Authenticate user
    if token != "#":
        try:
            response = requests.post(config.authentication_url(domain), headers={"Authorization": "Token {0}".format(token)})
            response.raise_for_status()
            user=json.loads(response._content.decode("utf-8"))
            if verbose:
                print("Logged in with username {0}".format(user['username']))
        except HTTPError:
            print("Invalid token: Credentials not found")
            sys.exit()
    else:
        token = prompt_token(domain)

    if verbose:
        print("\n\n***** Starting channel build process *****")

    # Set up progress tracker
    progress_manager = RestoreManager(debug)
    if (reset or not progress_manager.check_for_session()) and step.upper() != Status.DONE.name:
        progress_manager.init_session()
    else:
        if resume or prompt_resume():
            if verbose:
                print("Resuming your last session...")
            step = Status.LAST.name if step is None else step
            progress_manager = progress_manager.load_progress(step.upper())
        else:
            progress_manager.init_session()

    # Construct channel if it hasn't been constructed already
    if progress_manager.get_status_val() <= Status.CONSTRUCT_CHANNEL.value:
        channel = run_construct_channel(path, verbose, progress_manager, kwargs)
    else:
        channel = progress_manager.channel

    # Set initial tree if it hasn't been set already
    if progress_manager.get_status_val() <= Status.CREATE_TREE.value:
        tree = create_initial_tree(channel, domain, verbose, update, progress_manager, config.get_file_store())
    else:
        tree = progress_manager.tree

    # Download files if they haven't been downloaded already
    if progress_manager.get_status_val() <= Status.DOWNLOAD_FILES.value:
        process_tree_files(tree, verbose, progress_manager)
    else:
        tree.downloader.files = progress_manager.files_downloaded
        tree.downloader.failed_files = progress_manager.files_failed
        tree.downloader._file_mapping = progress_manager.file_mapping

    # Get file diff if it hasn't been generated already
    if progress_manager.get_status_val() <= Status.GET_FILE_DIFF.value:
        file_diff = get_file_diff(tree, verbose, progress_manager, token)
    else:
        file_diff = progress_manager.file_diff

    # Set which files have already been uploaded
    tree.uploaded_files = progress_manager.files_uploaded

    # Upload files if they haven't been uploaded already
    if progress_manager.get_status_val() <= Status.UPLOADING_FILES.value:
        upload_files(tree, file_diff, verbose, progress_manager, token)

    # Create channel on Kolibri Studio if it hasn't been created already
    if progress_manager.get_status_val() <= Status.UPLOAD_CHANNEL.value:
        channel_id, channel_link = create_tree(tree, verbose, progress_manager, token)
    else:
        channel_link = progress_manager.channel_link
        channel_id = progress_manager.channel_id

    # Publish tree if flag is set to True
    if publish and progress_manager.get_status_val() <= Status.PUBLISH_CHANNEL.value:
        publish_tree(tree, verbose, progress_manager, channel_id, token)

    # Open link on web browser (if specified) and return new link
    print("DONE: Channel created at {0}".format(channel_link))
    if prompt:
        prompt_open(channel_link)
    progress_manager.set_done()
    return channel_link

def prompt_token(domain):
    token = input("Enter authentication token ('q' to quit):").lower()
    if token == 'q':
        sys.exit()
    else:
        try:
            response = requests.post(config.authentication_url(domain), headers={"Authorization": "Token {0}".format(token)})
            response.raise_for_status()
            return response
        except HTTPError:
            print("Invalid token. Please login to {0}/settings/tokens to retrieve your authorization token.".format(domain))
            prompt_token(domain)

def prompt_resume():
    """ prompt_resume: Prompt user to resume last session if one exists
        Args: None
        Returns: None
    """
    openNow = input("Previous session detected. Would you like to resume your previous session? [y/n]:").lower()
    if openNow.startswith("y"):
        return True
    elif openNow.startswith("n"):
        return False
    else:
        return prompt_resume()

def run_construct_channel(path, verbose, progress_manager, kwargs):
    # Read in file to access create_channel method
    exec(open(path).read(), globals())

    # Create channel (using method from imported file)
    if verbose:
        print("Constructing channel...")
    channel = construct_channel(**kwargs)
    progress_manager.set_channel(channel)
    return channel

def create_initial_tree(channel, domain, verbose, update, progress_manager, file_store):

    # Create channel manager with channel data
    if verbose:
        print("Setting up initial channel structure...")
    tree = ChannelManager(channel, domain, file_store, verbose, update)

    # Create channel manager with channel data
    if verbose:
        print("Setting up node relationships..")
    tree.set_relationship(channel)

    # Make sure channel structure is valid
    if verbose:
        print("Validating channel structure...")
        channel.print_tree()
    tree.validate()
    progress_manager.set_tree(tree)
    return tree

def process_tree_files(tree, verbose, progress_manager):
    # Fill in values necessary for next steps
    if verbose:
        print("Processing content...")
    tree.process_tree(tree.channel)
    tree.check_for_files_failed()
    config.set_file_store(tree.downloader.file_store)
    progress_manager.set_files(tree.downloader.get_files(), tree.downloader.get_file_mapping(), tree.downloader.failed_files)

def get_file_diff(tree, verbose, progress_manager, token):
    # Determine which files have not yet been uploaded to the CC server
    if verbose:
        print("Checking if files exist on Kolibri Studio...")
    file_diff = tree.get_file_diff(token)
    progress_manager.set_diff(file_diff)
    return file_diff

def upload_files(tree, file_diff, verbose, progress_manager, token):
    # Upload new files to CC
    tree.upload_files(file_diff, progress_manager, token)
    progress_manager.set_uploaded(file_diff)

def create_tree(tree, verbose, progress_manager, token):
    # Create tree
    if verbose:
        print("Creating tree on Kolibri Studio...")
    channel_id, channel_link = tree.upload_tree(token)
    progress_manager.set_channel_created(channel_link, channel_id)
    return channel_id, channel_link

def prompt_open(channel_link):
    """ prompt_open: Prompt user to open web browser
        Args:
            channel_link (str): url of uploaded channel
        Returns: None
    """
    openNow = input("Would you like to open your channel now? [y/n]:").lower()
    if openNow.startswith("y"):
        print("Opening channel...")
        webbrowser.open_new_tab(channel_link)
    elif openNow.startswith("n"):
        return
    else:
        prompt_open(channel_link)

def publish_tree(tree, verbose, progress_manager, channel_id, token):
    """ publish_tree: Publish tree to Kolibri
        Args:
            channel (str): channel to publish
            token (str): authentication token
        Returns: None
    """
    if verbose:
        print("Publishing tree to Kolibri...")
    tree.publish(channel_id, token)
    progress_manager.set_published()



import os
import tempfile
import requests

with open('./../storage/f/2/f2912edc0aa959aef81093442ec715f0.jpg', 'rb') as tempf:
    response = requests.post('http://127.0.0.1:8000/api/internal/file_upload', headers={"Authorization": "Token {0}".format('9cab57ce6ee3b275afd7d1893f448a31da68f3a7')},  files={'file': tempf})
    response.raise_for_status()


# Managers to handle overall structures and api calls

import uuid
import hashlib
import json
import requests
import validators
import base64
import tempfile
import shutil
import pickle
import os
import sys
import filecmp
import requests
from enum import Enum
from requests_file import FileAdapter
from requests.exceptions import MissingSchema, HTTPError, ConnectionError, InvalidURL, InvalidSchema
from io import BytesIO
from PIL import Image
from ricecooker import config
from ricecooker.exceptions import InvalidFormatException, FileNotFoundException
from le_utils.constants import file_formats, exercises, format_presets

WEB_GRAPHIE_URL_REGEX = r'web\+graphie:([^\)]+)'
FILE_REGEX = r'!\[([^\]]+)?\]\(([^\)]+)\)'

class DownloadManager:
    """ Manager for handling file downloading and storage

        Attributes:
            session (Session): session to handle requests
            all_file_extensions ([str]): all accepted file extensions
            files ([str]): files that have been downloaded by download manager
            _file_mapping ([{'filename':{...}]): map from filename to file metadata
            verbose (bool): indicates whether to print what manager is doing (optional)
            update (bool): indicates whether to read from file paths again
    """

    # All accepted file extensions
    all_file_extensions = [key for key, value in file_formats.choices]

    def __init__(self, file_store, verbose=False, update=False):
        # Mount file:// to allow local path requests
        self.session = requests.Session()
        self.session.mount('file://', FileAdapter())
        self.update = update
        self.file_store = {}
        if os.stat(file_store).st_size > 0:
            with open(file_store, 'r') as jsonobj:
                self.file_store = json.load(jsonobj)
        self.files = []
        self.failed_files = []
        self._file_mapping = {} # Used to keep track of files and their respective metadata
        self.verbose = verbose

    def get_files(self):
        """ get_files: get files downloaded by download manager
            Args:None
            Returns: list of downloaded files
        """
        return self.files

    def get_file_mapping(self):
        """ get_file_mapping: get file metadata
            Args:None
            Returns: dict of file metadata
        """
        return self._file_mapping

    def has_failed_files(self):
        """ has_failed_files: check if any files have failed
            Args: None
            Returns: boolean indicating if files have failed
        """
        return len(self.failed_files) > 0

    def print_failed(self):
        """ print_failed: print out files that have failed downloading
            Args: None
            Returns: None
        """
        print("   The following files could not be accessed:")
        for f in self.failed_files:
            print("\t{id}: {path}".format(id=f[1], path=f[0]))

    def download_graphie(self, path, title):
        """ download_graphie: download a web+graphie file
            Args: path (str): path to .svg and .json files
            Returns: the combined hash of graphie files and their filenames
        """
        try:
            # Handle if path has already been processed
            if exercises.CONTENT_STORAGE_PLACEHOLDER in path:
                filename = os.path.split(path)[-1]
                return filename, filename + ".svg", filename + "-data.json"

            # Initialize paths and hash
            svg_path = path + ".svg"
            json_path = path + "-data.json"
            path_name = svg_path + ' & ' + json_path

            if self.check_downloaded_file(path_name):
                return self.track_existing_file(path_name)

            with tempfile.TemporaryFile(delete=False) as tempf:
                # Write graphie file
                self.write_to_graphie_file(svg_path, tempf)
                tempf.write(bytes(exercises.GRAPHIE_DELIMITER, 'UTF-8'))
                self.write_to_graphie_file(json_path, tempf)
                tempf.seek(0)

                graphie_result = self.download_file(
                    tempf.name,
                    title,
                    default_ext='.{}'.format(file_formats.GRAPHIE),
                    preset=format_presets.EXERCISE_GRAPHIE,
                    force_ext=True,
                    path_name=path_name,
                    original_filename=path.split("/")[-1].split(".")[0]
                )
                if not graphie_result:
                    raise FileNotFoundError("Could not access file: {0}".format(tempf.name))

            return graphie_result
        # Catch errors related to reading file path and handle silently
        except (HTTPError, FileNotFoundError, ConnectionError, InvalidURL, InvalidSchema, IOError):
            self.failed_files += [(path,title)]
            return False;

    def write_to_graphie_file(self, path, tempf):
        try:
            r = self.session.get(path, stream=True)
            r.raise_for_status()
            for chunk in r:
                tempf.write(chunk)
        except (MissingSchema, InvalidSchema):
            # Try opening path as relative file path
            with open(path, 'rb') as file_obj:
                tempf.write(file_obj.read())

    def get_hash(self, path):
        """ get_hash: generate hash of file
            Args:
                request (request): requested file
                hash_to_update (hash): hash to update based on file
            Returns: updated hash
        """
        hash_to_update = hashlib.md5()
        try:
            r = self.session.get(path, stream=True)
            r.raise_for_status()
            for chunk in r:
                hash_to_update.update(chunk)
        except (MissingSchema, InvalidSchema):
            with open(path, 'rb') as fobj:
                for chunk in iter(lambda: fobj.read(4096), b""):
                    hash_to_update.update(chunk)

        return hash_to_update

    def check_downloaded_file(self, path):
        return not self.update and path in self.file_store

    def track_existing_file(self, path):
        data = self.file_store[path]
        if self.verbose:
            print("\tFile {0} already exists (add '-u' flag to update)".format(data['filename']))
        self.track_file(data['filename'], data['size'],  data['preset'])
        return self._file_mapping[data['filename']]

    def download_file(self, path, title, default_ext=None, preset=None, force_ext=False, path_name=None, original_filename='file'):
        """ download_file: downloads file from path
            Args:
                path (str): local path or url to file to download
                default_ext (str): extension to use if none given (optional)
                preset (str): preset to use (optional)
                force_ext (bool): force manager to use default extension (optional)
            Returns: filename of downloaded file
        """
        try:
            path_name = path if path_name is None else path_name
            # Handle if path has already been processed
            if exercises.CONTENT_STORAGE_PLACEHOLDER in path:
                return os.path.split(path)[-1]

            if self.check_downloaded_file(path_name):
                return self.track_existing_file(path_name)

            if self.verbose:
                print("\tDownloading {}".format(path_name))

            hash=self.get_hash(path)

            # Get extension of file or default if none found
            file_components = path.split("/")[-1].split(".")
            extension = file_components[-1].lower()

            if force_ext or extension not in self.all_file_extensions:
                if default_ext is not None:
                    extension = default_ext
                else:
                    raise FileNotFoundError("No extension found: {}".format(path))
            else:
                extension = "." + extension
            filename = '{0}{ext}'.format(hash.hexdigest(), ext=extension)

            # If file already exists, skip it
            if os.path.isfile(config.get_storage_path(filename)):
                if self.verbose:
                    print("\t--- No changes detected on {0}".format(filename))

                # Keep track of downloaded file
                self.track_file(filename, os.path.getsize(config.get_storage_path(filename)), preset, path_name, original_filename)
                return self._file_mapping[filename]


            # Write file to temporary file
            with tempfile.TemporaryFile() as tempf:
                try:
                    # Access path
                    r = self.session.get(path, stream=True)
                    r.raise_for_status()

                    # Write to file (generate hash if none provided)
                    for chunk in r:
                        tempf.write(chunk)

                except (MissingSchema, InvalidSchema):
                    # If path is a local file path, try to open the file (generate hash if none provided)
                    with open(path, 'rb') as fobj:
                        tempf.write(fobj.read())

                # Get file metadata (hashed filename, original filename, size)
                file_size = tempf.tell()
                tempf.seek(0)

                # Keep track of downloaded file
                self.track_file(filename, file_size, preset, path_name, original_filename)

                # Write file to local storage
                with open(config.get_storage_path(filename), 'wb') as destf:
                    shutil.copyfileobj(tempf, destf)

                if self.verbose:
                    print("\t--- Downloaded '{0}' to {1}".format(original_filename, filename))
            return self._file_mapping[filename]
        # Catch errors related to reading file path and handle silently
        except (HTTPError, FileNotFoundError, ConnectionError, InvalidURL, InvalidSchema, IOError):
            self.failed_files += [(path,title)]
            return False;

    def track_file(self, filename, file_size, preset, path=None, original_filename='file'):
        self.files += [filename]
        file_data = {
            'size': file_size,
            'preset':preset,
            'filename':filename,
            'original_filename':original_filename,
        }
        self._file_mapping.update({filename : file_data})

        if path is not None:
            self.file_store.update({path:file_data})


    def download_files(self,files, title, default_ext=None):
        """ download_files: download list of files
            Args:
                files ([str]): list of file paths or urls to download
                title (str): name of node in case of error
            Returns: list of downloaded filenames
        """
        file_list = []
        for f in files:
            file_data = f.split('/')[-1]
            result = self.download_file(f, title, default_ext=default_ext)
            if result:
                file_list += [result]
        return file_list

    def encode_thumbnail(self, thumbnail):
        """ encode_thumbnail: gets base64 encoding of thumbnail
            Args:
                thumbnail (str): file path or url to channel's thumbnail
            Returns: base64 encoding of thumbnail
        """
        if thumbnail is None:
            return None
        else:
            # Check if thumbanil path is valid
            if validators.url(thumbnail):
                r = self.session.get(thumbnail, stream=True)
                if r.status_code == 200:
                    # Write thumbnail to temporary file
                    thumbnail = tempfile.TemporaryFile()
                    for chunk in r:
                        thumbnail.write(chunk)

            # Open image and resize accordingly
            img = Image.open(thumbnail)
            width = 200
            height = int((float(img.size[1])*float(width/float(img.size[0]))))
            img.thumbnail((width,height), Image.ANTIALIAS)

            # Write image to bytes for encoding
            bufferstream = BytesIO()
            img.save(bufferstream, format="PNG")
            return "data:image/png;base64," + base64.b64encode(bufferstream.getvalue()).decode('utf-8')

class ChannelManager:
    """ Manager for handling channel tree structure and communicating to server

        Attributes:
            channel (Channel): channel that manager is handling
            domain (str): server domain to create channel on
            downloader (DownloadManager): download manager for handling files
            verbose (bool): indicates whether to print what manager is doing (optional)
    """
    def __init__(self, channel, domain, file_store, verbose=False, update=False):
        self.channel = channel # Channel to process
        self.verbose = verbose # Determines whether to print process
        self.domain = domain # Domain to upload channel to
        self.update = update # Download all files if true

        self.downloader = DownloadManager(file_store, verbose, update)
        self.uploaded_files=[]

    def validate(self):
        """ validate: checks if tree structure is valid
            Args: None
            Returns: boolean indicating if tree is valid
        """
        return self.channel.test_tree()

    def set_relationship(self, node, parent=None):
        """ set_relationship: sets ids
            Args:
                node (Node): node to process
                parent (Node): parent of node being processed
            Returns: None
        """
        from ricecooker.classes import nodes

        # If node is not a channel, set ids and download files
        if not isinstance(node, nodes.Channel):
            node.set_ids(self.channel._internal_domain, parent.node_id)

        # Process node's children
        for child_node in node.children:
            self.set_relationship(child_node, node)

    def process_tree(self, node, parent=None):
        """ process_tree: processes files
            Args:
                node (Node): node to process
                parent (Node): parent of node being processed
            Returns: None
        """
        from ricecooker.classes import nodes

        # If node is not a channel, download files
        if not isinstance(node, nodes.Channel):
            node.files = self.downloader.download_files(node.files, "Node {}".format(node.original_id))
            if node.thumbnail is not None:
                result = self.downloader.download_files([node.thumbnail], "Node {}".format(node.original_id), default_ext=".{}".format(file_formats.PNG))
                if result:
                    node.files += result

            # If node is an exercise, process images for exercise
            if isinstance(node, nodes.Exercise):
                if self.verbose:
                    print("\t*** Processing images for exercise: {}".format(node.title))
                node.process_questions(self.downloader)

        # Process node's children
        for child_node in node.children:
            self.process_tree(child_node, node)

    def check_for_files_failed(self):
        if self.downloader.has_failed_files():
            self.downloader.print_failed()
        else:
            print("   All files were successfully downloaded")

    def get_file_diff(self, token):
        """ get_file_diff: retrieves list of files that do not exist on content curation server
            Args: None
            Returns: list of files that are not on server
        """
        files_to_diff = self.downloader.get_files()
        file_diff_result = []
        chunks = [files_to_diff[x:x+10000] for x in range(0, len(files_to_diff), 10000)]
        for chunk in chunks:
            response = requests.post(config.file_diff_url(self.domain), headers={"Authorization": "Token {0}".format(token)}, data=json.dumps(chunk))
            response.raise_for_status()
            file_diff_result += json.loads(response._content.decode("utf-8"))
        return file_diff_result

    def upload_files(self, file_list, progress_manager, token):
        """ upload_files: uploads files to server
            Args: file_list (str): list of files to upload
            Returns: None
        """
        counter = 0
        files_to_upload = list(set(file_list) - set(self.uploaded_files)) # In case restoring from previous session
        if self.verbose:
            print("Uploading {0} new file(s) to Kolibri Studio...".format(len(files_to_upload)))
        try:
            for f in files_to_upload:
                with  open(config.get_storage_path(f), 'rb') as file_obj:
                    response = requests.post(config.file_upload_url(self.domain), headers={"Authorization": "Token {0}".format(token)},  files={'file': file_obj})
                    response.raise_for_status()
                    self.uploaded_files += [f]
                    counter += 1
                    if self.verbose:
                        print("\tUploaded {0} ({count}/{total}) ".format(f, count=counter, total=len(files_to_upload)))
        finally:
            progress_manager.set_uploading(self.uploaded_files)

    def upload_tree(self, token):
        """ upload_files: sends processed channel data to server to create tree
            Args: None
            Returns: link to uploadedchannel
        """
        root, channel_id = self.add_channel(token)
        self.add_nodes(root, self.channel.children, token)
        return self.commit_channel(channel_id, token)

    def add_channel(self, token):
        """ upload_files: sends processed channel data to server to create tree
            Args: None
            Returns: link to uploadedchannel
        """
        payload = {
            "channel_data":self.channel.to_dict(),
        }
        response = requests.post(config.create_channel_url(self.domain), headers={"Authorization": "Token {0}".format(token)}, data=json.dumps(payload))
        response.raise_for_status()
        new_channel = json.loads(response._content.decode("utf-8"))
        return new_channel['root'], new_channel['channel_id']

    def add_nodes(self, root_id, children, token):
        payload = {
            'root_id': root_id,
            'content_data': [child.to_dict() for child in children]
        }
        response = requests.post(config.add_nodes_url(self.domain), headers={"Authorization": "Token {0}".format(token)}, data=json.dumps(payload))
        response.raise_for_status()

        response_json = json.loads(response._content.decode("utf-8"))

        for child in children:
            self.add_nodes(response_json['root_ids'][child.node_id.hex], child.children, token)

    def commit_channel(self, channel_id, token):
        """ commit_channel: commits channel to Kolibri Studio
            Args: None
            Returns: link to uploadedchannel
        """
        payload = {
            "channel_id":channel_id,
        }
        response = requests.post(config.finish_channel_url(self.domain), headers={"Authorization": "Token {0}".format(token)}, data=json.dumps(payload))
        response.raise_for_status()
        new_channel = json.loads(response._content.decode("utf-8"))
        return channel_id, config.open_channel_url(new_channel['new_channel'], self.domain)

    def publish(self, channel_id, token):
        """ publish: publishes tree to Kolibri
            Args: None
            Returns: link to uploadedchannel
        """
        payload = {
            "channel_id":channel_id,
        }
        response = requests.post(config.publish_channel_url(self.domain), headers={"Authorization": "Token {0}".format(token)}, data=json.dumps(payload))
        response.raise_for_status()

class Status(Enum):
    INIT = 0
    CONSTRUCT_CHANNEL = 1
    CREATE_TREE = 2
    DOWNLOAD_FILES = 3
    GET_FILE_DIFF = 4
    START_UPLOAD = 5
    UPLOADING_FILES = 6
    UPLOAD_CHANNEL = 7
    PUBLISH_CHANNEL = 8
    DONE = 9
    LAST = 10


class RestoreManager:
    """ Manager for handling resuming rice cooking process

        Attributes:
            restore_path (str): path to .pickle file to store progress
    """

    def __init__(self, debug):
        self.debug = debug
        self.channel = None
        self.tree = None # Tree to process
        self.files_downloaded = [] # Determines whether to print process
        self.file_mapping = {} # Domain to upload channel to
        self.files_failed = [] # Download all files if true
        self.file_diff = []
        self.files_uploaded = []
        self.channel_link = None
        self.channel_id = None
        self.status = Status.INIT

    def check_for_session(self, status=None):
        status = Status.LAST if status is None else status
        return os.path.isfile(self.get_restore_path(status)) and os.path.getsize(self.get_restore_path(status)) > 0

    def get_restore_path(self, status=None):
        status = self.get_status() if status is None else status
        return config.get_restore_path(status.name.lower(), self.debug)

    def record_progress(self):
        with open(self.get_restore_path(Status.LAST), 'wb') as handle, open(self.get_restore_path(), 'wb') as step_handle:
            pickle.dump(self, handle)
            pickle.dump(self, step_handle)

    def load_progress(self, resume_step):
        resume_step = Status[resume_step]
        progress_path = self.get_restore_path(resume_step)

        # If progress is corrupted, revert to step before
        while not self.check_for_session(resume_step):
            print("Ricecooker has not reached {0} status. Reverting to earlier step...".format(resume_step.name))
            # All files are corrupted, restart process
            if resume_step.value - 1 < 0:
                self.init_session()
                return self
            resume_step = Status(resume_step.value - 1)
            progress_path = self.get_restore_path(resume_step)
        print("Starting from status {0}".format(resume_step.name))

        with open(progress_path, 'rb') as handle:
            manager = pickle.load(handle)
            if isinstance(manager, RestoreManager):
                return manager
            else:
                return self

    def get_status(self):
        return self.status

    def get_status_val(self):
        return self.status.value

    def init_session(self):
        for status in Status:
            path = self.get_restore_path(status)
            if os.path.isfile(path):
                os.remove(path)
        self.record_progress()
        self.status = Status.CONSTRUCT_CHANNEL # Set status to next step
        self.record_progress()

    def set_channel(self, channel):
        self.status = Status.CREATE_TREE # Set status to next step
        self.channel = channel
        self.record_progress()

    def set_tree(self, tree):
        self.status = Status.DOWNLOAD_FILES # Set status to next step
        self.tree = tree
        self.record_progress()

    def set_files(self, files_downloaded, file_mapping, files_failed):
        self.status = Status.GET_FILE_DIFF # Set status to next step
        self.files_downloaded = files_downloaded
        self.file_mapping = file_mapping
        self.files_failed = files_failed
        self.record_progress()

    def set_diff(self, file_diff):
        self.status = Status.START_UPLOAD # Set status to next step
        self.file_diff = file_diff
        self.record_progress()

    def set_uploading(self, files_uploaded):
        self.status = Status.UPLOADING_FILES
        self.files_uploaded = files_uploaded
        self.record_progress()

    def set_uploaded(self, files_uploaded):
        self.files_uploaded = files_uploaded
        self.status = Status.UPLOAD_CHANNEL # Set status to next step
        self.record_progress()

    def set_channel_created(self, channel_link, channel_id):
        self.status = Status.PUBLISH_CHANNEL
        self.channel_link = channel_link
        self.channel_id = channel_id
        self.record_progress()

    def set_published(self):
        self.status = Status.DONE
        self.record_progress()

    def set_done(self):
        self.status = Status.DONE
        self.record_progress()
        os.remove(self.get_restore_path(Status.LAST))



# Settings for rice cooker

import os
import json

# Domain for uploading to production server
PRODUCTION_DOMAIN = "https://contentworkshop.learningequality.org"

# Domain for uploading to local machine
DEBUG_DOMAIN = "http://127.0.0.1:8000"

# URL for authenticating user on Kolibri Studio
AUTHENTICATION_URL = "{domain}/api/internal/authenticate_user_internal"

# URL for getting file diff
FILE_DIFF_URL = "{domain}/api/internal/file_diff"

# URL for uploading files to server
FILE_UPLOAD_URL = "{domain}/api/internal/file_upload"

# URL for creating channel on server
CREATE_CHANNEL_URL = "{domain}/api/internal/create_channel"

# URL for adding nodes to channel
ADD_NODES_URL = "{domain}/api/internal/add_nodes"

# URL for making final changes to channel
FINISH_CHANNEL_URL = "{domain}/api/internal/finish_channel"

# URL to return after channel is created
OPEN_CHANNEL_URL = "{domain}/channels/{channel_id}/edit"

# URL for publishing channel
PUBLISH_CHANNEL_URL = "{domain}/api/internal/publish_channel"

# Folder to store downloaded files
STORAGE_DIRECTORY = "storage"

# Folder to store progress tracking information
RESTORE_DIRECTORY = "restore"

def get_storage_path(filename):
    """ get_storage_path: returns path to storage directory for downloading content
        Args: filename (str): Name of file to store
        Returns: string path to file
    """
    directory = os.path.join(STORAGE_DIRECTORY, filename[0], filename[1])
    # Make storage directory for downloaded files if it doesn't already exist
    if not os.path.exists(directory) :
        os.makedirs(directory)

    return os.path.join(directory, filename)

def authentication_url(domain):
    """ authentication_url: returns url to login to Kolibri Studio
        Args: domain (str): domain to log in
        Returns: string url to authenticate_user_internal endpoint
    """
    return AUTHENTICATION_URL.format(domain=domain)

def init_file_mapping_store(debug):
    """ init_file_mapping_store: creates log to keep track of downloaded files
        Args: debug (bool): determines where to store progress information
        Returns: None
    """
    # Make storage directory for restore files if it doesn't already exist
    path = os.path.join(RESTORE_DIRECTORY, "local" if debug else "production")
    if not os.path.exists(path):
        os.makedirs(path)

    # Create file mapping json if it doesn't exist
    path = os.path.join(RESTORE_DIRECTORY, "file_restore.json")
    if not os.path.isfile(path):
        open(path, 'a').close()

def get_file_store():
    """ get_file_store: returns path to list of downloaded files
        Args: None
        Returns: string path to list of downloaded files
    """
    return os.path.join(RESTORE_DIRECTORY, "file_restore.json")

def set_file_store(file_store):
    """ set_file_store: saves list of downloaded files
        Args: file_store ([{path: {size:number, preset:str, filename:str, original_filename:str}}]): list of downloaded files in json format
        Returns: None
    """
    with open(get_file_store(), 'w') as storeobj:
        json.dump(file_store, storeobj)

def get_restore_path(filename, debug):
    """ get_restore_path: returns path to directory for restoration points
        Args:
            filename (str): Name of file to store
            debug (bool): Determines how to store restoration points
        Returns: string path to file
    """
    path = os.path.join(RESTORE_DIRECTORY, "local" if debug else "production")
    return os.path.join(path, filename + '.pickle')


def file_diff_url(domain):
    """ file_diff_url: returns url to get file diff
        Args: domain (str): domain to get file diff from
        Returns: string url to file_diff endpoint
    """
    return FILE_DIFF_URL.format(domain=domain)

def file_upload_url(domain):
    """ file_upload_url: returns url to upload files
        Args: domain (str): domain to upload files to
        Returns: string url to file_upload endpoint
    """
    return FILE_UPLOAD_URL.format(domain=domain)

def create_channel_url(domain):
    """ create_channel_url: returns url to create channel
        Args: domain (str): domain to create channel on
        Returns: string url to create_channel endpoint
    """
    return CREATE_CHANNEL_URL.format(domain=domain)

def add_nodes_url(domain):
    """ add_nodes_url: returns url to add nodes to channel
        Args: domain (str): domain to create channel on
        Returns: string url to add_nodes endpoint
    """
    return ADD_NODES_URL.format(domain=domain)

def finish_channel_url(domain):
    """ finish_channel_url: returns url to finish uploading a channel
        Args: domain (str): domain to create channel on
        Returns: string url to finish_channel endpoint
    """
    return FINISH_CHANNEL_URL.format(domain=domain)

def open_channel_url(channel, domain):
    """ open_channel_url: returns url to uploaded channel
        Args:
            channel (str): channel id of uploaded channel
            domain (str): server where channel was created
        Returns: string url to open channel
    """
    return OPEN_CHANNEL_URL.format(domain=domain, channel_id=channel)

def publish_channel_url(domain):
    """ open_channel_url: returns url to publish channel
        Args:
            domain (str): server where channel was created
        Returns: string url to publish channel
    """
    return PUBLISH_CHANNEL_URL.format(domain=domain)
